// as per https://www.ietf.org/archive/id/draft-gallagher-openpgp-hkp-05.html#name-machine-readable-indexes

export interface Index {
    prefix: 'info' | 'pub' | 'uid'
}

export type Indexes = Index[]

export class InfoLine implements Index {
    prefix: 'info'
    version: 1
    count: number

    constructor(indexLine: string) {
        parseIndex<InfoLine>(this, indexLine, ['version', 'count'])

        this.prefix = 'info'

        if (this.version != 1) {
            throw new Error("InfoLine's version MUST be 1! Got " + this.version)
        }
        if (this.count) {
            if (typeof this.count !== 'number') {
                if (isNaN(this.count)) {
                    throw new Error("InfoLine's count MUST NOT be NaN!")
                }
            }
        }
    }
}

export class PubLine implements Index {
    prefix: 'pub'
    keyid?: string
    keylen?: number
    algorithm?: string
    creationdate?: number
    expirationdate?: number
    flags?: string
    version?: string

    constructor(indexLine: string) {
        parseIndex<PubLine>(this, indexLine, [
            'keyid',
            'keylen',
            'algorithm',
            'creationdate',
            'expirationdate',
            'flags',
            'version',
        ])

        this.prefix = 'pub'

        if (this.creationdate && typeof this.creationdate !== 'number') {
            this.creationdate = parseFloat(this.creationdate)
            if (isNaN(this.creationdate)) {
                throw new Error("PubLine's creationdate MUST NOT be NaN!")
            }
        }
        if (this.expirationdate && typeof this.expirationdate !== 'number') {
            this.expirationdate = parseFloat(this.expirationdate)
            if (isNaN(this.expirationdate)) {
                throw new Error("PubLine's expirationdate MUST NOT be NaN!")
            }
        }
    }
}

export class UidLine implements Index {
    prefix: 'uid'
    uidstring?: string
    creationdate?: string
    expirationdate?: string
    flags?: string

    constructor(indexLine: string) {
        parseIndex<UidLine>(this, indexLine, [
            'uidstring',
            'creationdate',
            'expirationdate',
            'flags',
        ])

        this.prefix = 'uid'
    }
}

export function assertValidPrefix(prefix: string, throwError = true): boolean {
    if (['info', 'pub', 'uid'].indexOf(prefix) == -1) {
        if (!throwError) {
            return false
        }
        throw new Error('Prefix must be one of: info, pub, uid')
    }
    return true
}

export function parseIndexes(untyped: string[]): Indexes {
    return untyped
        .filter((x) => x.split(':').length > 1)
        .filter((x) => assertValidPrefix(x.split(':')[0], false))
        .map((x) => {
            const prefix = x.split(':')[0] as 'info' | 'pub' | 'uid'
            switch (prefix) {
                case 'info':
                    return new InfoLine(x)
                case 'pub':
                    return new PubLine(x)
                case 'uid':
                    return new UidLine(x)
            }
        })
}

export function parseIndex<T extends Index>(
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    self: any,
    index: string,
    keys: (keyof T)[]
): void {
    let exploded = index.replaceAll('\r', '').split(':')

    if (keys.length > exploded.length) {
        throw new Error('keys MUST NOT be longer than index')
    }

    const prefix = exploded[0] as 'info' | 'pub' | 'uid'
    assertValidPrefix(prefix, true)
    self.prefix = prefix
    exploded = exploded.slice(1, exploded.length)

    for (let i = 0; i != keys.length; i++) {
        self[keys[i]] = decodeURIComponent(exploded[i])
    }
}

export function serializeIndexes(indexes: Indexes): string {
    const out: (string | number | undefined)[][] = []
    for (const index of indexes) {
        if (index instanceof InfoLine) {
            out.push(['info', index.version, index.count])
        }
        if (index instanceof PubLine) {
            out.push([
                'pub',
                index.keyid,
                index.algorithm,
                index.keylen,
                index.creationdate,
                index.expirationdate,
                index.flags,
                index.version,
            ])
        }
        if (index instanceof UidLine) {
            out.push([
                'uid',
                index.uidstring,
                index.creationdate,
                index.expirationdate,
                index.flags,
            ])
        }
    }

    return (
        out
            .map((x) =>
                x
                    .map((x) => x ?? '')
                    .map(encodeURIComponent)
                    .join(':')
            )
            .join('\n') + '\n'
    )
}
